#include <errno.h>
#include <fcntl.h>
#include <ioctl.h>
#include <mintbind.h>
#include <stdio.h>
#include <signal.h>
#include "syslog/syslog.h"
#include "lpr.h"


void printjob(signo)
int signo;
{
	FILE	*fd;
	Lpr_d	dt;
	char	quit = 0;

	do {
		get_semaphore();
		openlog("lpd", 0, LOG_LPR);
		if ((fd=fopen(LPRSTAT,"rb")) == (FILE *)0) {
			syslog(LOG_ERR, "can't open LPRSTAT");
			closelog();
			destroy_semaphore();
			exit(0);
		}
		fseek(fd,(long)sizeof(Lpr_h),0);
		if (fread(&dt,sizeof(Lpr_d),1,fd) == 1) {
			fclose(fd);
			free_semaphore();
			print(dt.job,dt.copies);
		} else
			quit = 1;
	} while (quit == 0);
	fclose(fd);
	free_semaphore();
}

void terminate(signo)
int signo;
{
	FILE	*fd;
	Lpr_h	hd;

	get_semaphore();
	fd = fopen(LPRSTAT,"rb+");
	fread(&hd,sizeof(hd),1,fd);
	hd.lpd = DIED;
	rewind(fd);
	fwrite(&hd,sizeof(hd),1,fd);
	fclose(fd);
	openlog("lpd", 0, LOG_LPR);
	syslog(LOG_NOTICE,"caught signal %d",signo);
	closelog();
	destroy_semaphore();
	exit(0);
}

int main(argc,argv)
int argc;
char *argv[];
{
	FILE	*fd, *err;
	Lpr_h	hd;
	long	seks;
	int		i;
	void	terminate();
	void	printjob();

#ifndef NODEAMON
	switch ( fork()) {
		case 0:
			/* This is the new deamon process */
#ifdef OLD_MINT
			/* kill the parent process, because it's stoped */
			Pkill(Pgetppid(),SIGKILL);
#endif
			break;
		case ENOMEM:
			printf("Not enough space!!!\n");
			exit( ENOMEM);	/* Error appeared */
			break;
		default:
			exit( 0);	/* exit the fatherprocess */
	}
#endif

	umask( 066);		/* set filemask */
	chdir( "/var/spool");/* change CurrentWorkDirectory */
	setpgrp();			/* change process-group */

	for (i = 0; i < 10; i++)
		(void) close(i);/* close all open files */

	(void) open("/", 0);/* redirect the std-channels */ 
	(void) dup2(0, 1);
	(void) dup2(0, 2);

	if ((open("/dev/tty", O_RDWR)) >= 0) {
		(void) ioctl(i, (int) TIOCNOTTY, (char *)0);
		(void) close(i);
	}					/* free from tty; change to /dev/null */

	signal( SIGTERM, terminate);
	signal( SIGKILL, terminate);
	signal( SIGHUP, printjob);

	create_semaphore();
	openlog("lpd", 0, LOG_LPR);

	time( &seks);

	if ((fd=fopen(LPRSTAT,"rb+")) == (FILE *)0) {
		syslog(LOG_NOTICE,"started: %s",ctime(&seks));
		if ((fd=fopen(LPRSTAT,"wb")) == (FILE *)0) {
			syslog(LOG_ERR, "error: can't create LPRSTAT");
			closelog();
			exit(-1);
		}
		fclose(fd);
		fd = fopen(LPRSTAT,"rb+");

	} else {

		if (fread(&hd,sizeof(Lpr_h),1,fd) != 1) {
			syslog(LOG_ERR, "error: can't read LPRSTAT");
			closelog();
			fclose(fd);
			exit(-1);
		}
		if (hd.lpd == EXISTS) {
			syslog(LOG_ERR, "error: it's already running a daemon");
			closelog();
			fclose(fd);
			exit(0);
		}
		syslog(LOG_NOTICE,"started: %s",ctime(&seks));
	}

	closelog();
	hd.lpd = EXISTS;
	hd.pid = getpid();
	rewind(fd);
	fwrite(&hd,sizeof(hd),1,fd);
	rewind(fd);
	fseek(fd,(long)sizeof(hd),0);
	fclose(fd);
	free_semaphore();
	while (1) Pause();	/* wait for signals */
}
