#include <ctype.h>
#include <limits.h>
#include <memory.h>
#include <osbind.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "lpr.h"

int  header 	= 0;
int  len		= 60;
int  formfeed	= 0;
int  nummer 	= 0;
int  loeschen	= 0;
int  translation = 0;
int  copy		= 1;
int  tanz		= 0;
int  lpd_id		= INT_MAX;

Table liste[256][512];

void parsing(file)
char *file;
{
	FILE	*fd;
	char	zeile[256],*z;
	int 	zeiint;
	int 	comment;

	if ((fd=fopen(file,"r")) == (FILE *)0) {
		printf("can't open translation table %s\n",file);
		printf("can't translate any character....\n");
		translation = 0;
		return;
	}
	while (fgets(zeile,256,fd) != NULL) {
#ifdef DEBUG
		printf("parsing line: %s\n",zeile);
#endif	   
		zeiint = 0;
		liste[tanz][0].anz = 1;
		for (z=zeile; *z; z++) {
			comment = 0;
			switch (*z) {
				case '#': /* comment found: rest of line is comment */
					comment = 1;
					break;
				case ':': /* ASCII-Code of char, */
						  /* who's to translate, ending */
					liste[tanz][0].c = (char )zeiint;
#ifdef DEBUG
					printf("Char who's to translate: %d\n",zeiint);
#endif
					zeiint = 0;
					break;
				case ';': /* ending of translation sequenze */
						  /* rest of line is comment :-) */
					liste[tanz][liste[tanz][0].anz].c = (char )zeiint;
					liste[tanz][0].anz++;
#ifdef DEBUG
					printf("found translate (%d.)char: %d\n",
					liste[tanz][0].anz,zeiint);
#endif
					zeiint = 0;
					tanz++;
					comment = 1;
					break;
				case ',': /* ending of one translate-char */
					liste[tanz][liste[tanz][0].anz].c = (char )zeiint;
					liste[tanz][0].anz++;
#ifdef DEBUG
					printf("found translate (%d.)char: %d\n",
					liste[tanz][0].anz,zeiint);
#endif
					zeiint = 0;
					break;
				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					zeiint *= 10;
					zeiint += ((int)*z - (int)'0');
					break;
				default :
					break;
			}
			if (comment == 1) break;
		} /* end of for */
	} /* end while(fgets) */
	fclose(fd);
} /* end of parsing */

void source(file)
char *file;
{
	FILE	*queue;
	char	path[132];
	long	seks;
	long	filelen;
	struct	tm *now;
	Lpr_h	hd;
	Lpr_d	dt;

	get_semaphore();	 /* Lock Queue-File through semaphore */
	if ((queue = fopen(LPRSTAT,"rb+")) == (FILE *)0) {
		printf("lpr: can't open LPRSTAT\n");
		free_semaphore();
		exit(-1);
	}
	if (fread(&hd,sizeof(Lpr_h),1,queue) != 1) {
		printf("lpr: can't read LPRSTAT\n");
		fclose(queue);
		free_semaphore();
		exit(-1);
	}
	if (hd.lpd != EXISTS) {		/* No Daemon running	  */
		printf("lpr: no lpd running ?!\n");
		fclose(queue);
		free_semaphore();
		exit(-1);
	}
	lpd_id = hd.pid;
	fseek(queue,0L,2);
	filelen = ftell(queue);
	if ((filelen - sizeof(hd)) == 0) {
		dt.job = 0;
	} else {
		fseek(queue,filelen - sizeof(dt),0);
		if (fread(&dt,sizeof(dt),1,queue) != 1) {
			dt.job = 0;
		}
	}
	dt.job++;
	sprintf(path,"u:\\var\\spool\\A%ld",dt.job);
	sprintf(dt.owner,"%s",getlogin());
	dt.copies = copy;
	time(&seks);
	now = localtime(&seks);
	sprintf(dt.date,"%02d.%02d.%04d %02d:%02d", now->tm_mday,now->tm_mon + 1,
		now->tm_year + 1900, now->tm_hour, now->tm_min);
	sprintf(dt.file,"%s",file);
	fseek(queue,0L,2);
	if ( fwrite(&dt,sizeof(dt),1,queue) != 1) {
		printf("lpr: can't write LPRSTAT\n");
		fclose(queue);
		free_semaphore();
		exit(-1);
	}
	fclose(queue);
	free_semaphore();

	if ( (header + nummer + translation)) {
		write_file( file, path);
	} else {
		copy_file( file, path);
	}
	
	if (loeschen == 1) {
#ifdef DEBUG
		printf("i've now removing the file\n");
#endif
		printf("removing %s....\n",file);
		unlink(file);
	}
}

void usage( void)
{
	printf("usage: lpr [-hnfr] [-l <nr>] [-t <t_table>] [-c <nr>] file ....\n");
	printf("-h: print header-line\n");
	printf("-n: numbering lines\n");
	printf("-f: after file print a formfeed\n");
	printf("-r: after print removing file\n");
	printf("-l <nr> : set paper-lenght to <nr>\n");
	printf("-t <translation table> : load translation table\n");
	printf("-c <nr> : make n copies of the file\n");
	exit(-1);
}

int main(argc,argv)
int argc;
char *argv[];
{
	int 	i;
	char	*flags;

	if (argc < 2) usage();
	for (i=1; i < argc; i++) {
#ifdef DEBUG
		printf("parsing %d. argument(%s)\n",i,argv[i]);
#endif
		if (argv[i][0] == '-') {
#ifdef DEBUG
			printf("found flags\n");
#endif
			flags = &argv[i][1];
			for (;*flags; flags++) {
#ifdef DEBUG
				printf("found flagsymbol %c\n",*flags);
#endif
				switch (*flags) {
					case 'h':
					case 'H':
#ifdef DEBUG
						printf("setting header\n");
#endif
						header = 1;
						break;
					case 'r':
					case 'R':
#ifdef DEBUG
						printf("setting removing\n");
#endif
						loeschen = 1;
						break;
					case 'f':
					case 'F':
#ifdef DEBUG
						printf("setting formfeed\n");
#endif
						formfeed = 1;
						break;
					case 'n':
					case 'N':
#ifdef DEBUG
						printf("setting numbering\n");
#endif
						nummer = 1;
						break;
					case 't':
					case 'T':
						parsing(argv[++i]);
#ifdef DEBUG
						printf("setting translation\n");
						printf("translation table: %s\n",argv[i]);
#endif
						translation = 1;
						break;
					case 'l':
					case 'L':
#ifdef DEBUG
						printf("setting len\n");
#endif
						len = atoi(argv[++i]);
						break;
					case 'c' :
					case 'C' :
#ifdef DEBUG
						printf("setting copies\n");
#endif
						copy = atoi(argv[++i]);
						copy = (copy != 0 ? copy : 1);
						break;
					default :
						usage();
						exit(0);
				} /* end of switch */
			} /* end of for */
		} /* end of if */
		else source(argv[i]);
	} /* end of for */

/* new signal stuff
 * send now a HUP-Signal to wake up the lpd */
	kill( lpd_id, SIGHUP);
	exit(0);
} /* end of main */
